#pragma once

#include <random>
#include <fstream>
#include <string>
#include <sstream>
#include <iostream>
#include <iterator>
#include <algorithm>

void CreateDataStream(const std::string &fileName);
void CreateDataGenerate(const std::string &fileName);

template<class ContainerType>
ContainerType LoadDataStream(const std::string &fileName) {
  ContainerType buffer;
  using ValueType = typename ContainerType::value_type;
  std::ifstream fileRead(fileName);
  std::string bufferLine;
  while (std::getline(fileRead, bufferLine, '\t')) {
    std::istringstream iss(bufferLine);
    ValueType elem;
    if (!(iss >> elem)) {
      break;
    }
    buffer.push_back(elem);
  }
  fileRead.close();
  return buffer;
}

template<class ContainerType>
ContainerType LoadDataCopy(const std::string &fileName) {
  using ValueType = typename ContainerType::value_type;
  ContainerType buffer;
  std::ifstream fileRead(fileName);
  std::copy(std::istream_iterator<ValueType>(fileRead), std::istream_iterator<ValueType>(), std::back_inserter(buffer));
  fileRead.close();
  return buffer;
}

template<typename ContainerType>
void ModifyTemplate(ContainerType &modifyContainer) {
  using ValueType = typename ContainerType::value_type;
  double sum = 0;
  size_t elemCount = 0;
  auto firstNegativeIter =
      std::find_if(modifyContainer.begin(), modifyContainer.end(), [](const ValueType &elem) { return elem < 0; });
  ValueType firstNegative = *firstNegativeIter;
  if (firstNegativeIter != modifyContainer.end()) {
    for (ValueType &elem: modifyContainer) {
      sum += std::abs(elem);
      ++elemCount;
      if (elem % 2 == 0) {
        elem *= firstNegative;
      }
    }
  }

  modifyContainer.push_back(sum);
  if (elemCount != 0) {
    modifyContainer.push_back(sum / elemCount);
  }
}

template<typename Iterator>
void ModifyIter(const Iterator iterBegin, const Iterator iterEnd) {
  double sum = 0;
  size_t elemCount = 0;
  using ValueType = typename Iterator::value_type;
  auto firstNegativeIter =
      std::find_if(iterBegin, iterEnd, [](const ValueType &elem) { return elem < 0; });
  ValueType firstNegative = *firstNegativeIter;
  if (firstNegativeIter != iterEnd) {
    for (auto it = iterBegin; it != iterEnd; ++it) {
      sum += std::abs(*it);
      ++elemCount;
      if (*it % 2 == 0) {
        *it *= firstNegative;
      }
    }
  }
}

template<typename ContainerType>
void ModifyForEach(ContainerType &cont) {
  using ValueType = typename ContainerType::value_type;
  auto firstNegativeIter = std::find_if(cont.begin(), cont.end(), [](const ValueType &elem) { return elem < 0; });
  ValueType firstNegative = *firstNegativeIter;
  std::for_each(cont.begin(), cont.end(), [&firstNegative](ValueType &elem) {
    if (elem % 2 == 0) {
      elem *= firstNegative;
    }
  });
}

void ModifyTransform(const std::string &inputFileName, const std::string &OutputFileName);

template<typename ContainerType>
void OutputResultStream(const std::string &fileName, const ContainerType &outputContainer) {
  std::ofstream fileOutputStream(fileName, std::ios::out);
  for (const auto &elem: outputContainer) {
    fileOutputStream << elem << "\t";
  }
  fileOutputStream << std::endl;
}

template<typename ContainerType>
void OutputResultCopy(const std::string &fileName, const ContainerType &outputContainer) {
  using ValueType = typename ContainerType::value_type;
  std::ofstream fileOutputStream(fileName, std::ios::out);
  std::copy(outputContainer.begin(), outputContainer.end(), std::ostream_iterator<ValueType>(fileOutputStream, "\t"));
  fileOutputStream << std::endl;
}

