#include "db_person.h"

DBPerson::DBPerson(const std::string &filename) : filename(filename) {

}
void DBPerson::Open() {
  std::ofstream fileOutputStream;
  fileOutputStream.open(filename, std::ios::binary | std::ios::out);

  std::cout << "Enter new persons:" << std::endl;
  std::cout
      << "| " << std::setw(13) << "Person number"
      << " | " << std::setw(17) << "Department number"
      << " | " << std::setw(11) << "Name"
      << " | " << std::setw(11) << "Surname"
      << " | " << std::setw(11) << "Patronymic"
      << " | " << std::setw(6) << "Salary"
      << " | " << std::setw(2) << "dayStart"
      << " | " << std::setw(2) << "monthStart"
      << " | " << std::setw(4) << "yearStart"
      << " | " << std::setw(2) << "premiumPercent"
      << " | " << std::setw(6) << "tax"
      << " | " << std::setw(2) << "workDaysCountDone"
      << " | " << std::setw(2) << "workDaysCount"
      << " | " << std::setw(6) << "received"
      << " | " << std::setw(6) << "withheld"
      << " | " << std::setw(6) << "gender"
      << " |" << std::endl;

  Person person{};

  while (true) {
    std::cout << "Person:\n";
    std::cin >> person;
    fileOutputStream.write(reinterpret_cast<const char *>(&person), sizeof(person));
    std::cout << "0 - End input\n"
              << "1 - Add more persons\n";
    int result;
    std::cin >> result;

    if (result == 0) {
      break;
    }
  }

  fileOutputStream.close();
}
void DBPerson::LoadData() {
  std::ifstream dbInput(filename, std::ios::binary | std::ios::in);
  Person currentPerson{};
  int id = 0;
  while (dbInput.read(reinterpret_cast<char *>(&currentPerson), sizeof(currentPerson))) {
    dataTable.insert(std::make_pair(++id, currentPerson));
  }
  dbInput.close();
}

void DBPerson::SelectByDepartmentNumber(const std::string &savefilename, int departmentNumber) {
  LoadData();
  std::ofstream filterFileOutStream(savefilename, std::ios::binary | std::ios::out);
  auto filterOperation = [departmentNumber, &filterFileOutStream]
      (const std::pair<int, Person> &current) {
    if (current.second.departmentNumber == departmentNumber) {
      filterFileOutStream.write(reinterpret_cast<const char *>(&current.second), sizeof(Person));
    }
  };

  std::for_each(dataTable.begin(),
                dataTable.end(),
                filterOperation);
  filterFileOutStream.close();
}
void DBPerson::SelectByGender(const std::string &savefilename, bool gender) {
  LoadData();
  std::ofstream filterFileOutStream(savefilename, std::ios::binary | std::ios::out);
  auto filterOperation = [gender, &filterFileOutStream]
      (const std::pair<int, Person> &current) {
    if (current.second.gender == gender) {
      filterFileOutStream.write(reinterpret_cast<const char *>(&current.second), sizeof(Person));
    }
  };

  std::for_each(dataTable.begin(),
                dataTable.end(),
                filterOperation);
  filterFileOutStream.close();
}
void DBPerson::SelectByDateStart(const std::string &savefilename, int dayStart, int monthStart, int yearStart) {
  LoadData();
  std::ofstream filterFileOutStream(savefilename, std::ios::binary | std::ios::out);
  auto filterOperation = [dayStart, monthStart, yearStart, &filterFileOutStream]
      (const std::pair<int, Person> &current) {
    if (current.second.dayStart == dayStart &&
        current.second.monthStart == monthStart &&
        current.second.yearStart == yearStart) {
      filterFileOutStream.write(reinterpret_cast<const char *>(&current.second), sizeof(Person));
    }
  };

  std::for_each(dataTable.begin(),
                dataTable.end(),
                filterOperation);
  filterFileOutStream.close();
}
void DBPerson::SerializeText() {
  LoadData();
  std::ofstream outputFileStream(filename + ".txt", std::ios::out);
  outputFileStream
      << "| " << std::setw(13) << "Person number"
      << " | " << std::setw(17) << "Department number"
      << " | " << std::setw(11) << "Name"
      << " | " << std::setw(11) << "Surname"
      << " | " << std::setw(11) << "Patronymic"
      << " | " << std::setw(6) << "Salary"
      << " | " << std::setw(8) << "dayStart"
      << " | " << std::setw(10) << "monthStart"
      << " | " << std::setw(9) << "yearStart"
      << " | " << std::setw(14) << "premiumPercent"
      << " | " << std::setw(6) << "tax"
      << " | " << std::setw(17) << "workDaysCountDone"
      << " | " << std::setw(13) << "workDaysCount"
      << " | " << std::setw(8) << "received"
      << " | " << std::setw(8) << "withheld"
      << " | " << std::setw(6) << "gender"
      << " |" << std::endl;

  auto serializeRowOperation = [&outputFileStream]
      (const std::pair<int, Person> &current) {
    outputFileStream
        << "| " << std::setw(13) << current.second.personNumber
        << " | " << std::setw(17) << current.second.departmentNumber
        << " | " << std::setw(11) << current.second.name
        << " | " << std::setw(11) << current.second.surname
        << " | " << std::setw(11) << current.second.patronymic
        << " | " << std::setw(6) << current.second.salary
        << " | " << std::setw(8) << current.second.dayStart
        << " | " << std::setw(10) << current.second.monthStart
        << " | " << std::setw(9) << current.second.yearStart
        << " | " << std::setw(14) << current.second.premiumPercent
        << " | " << std::setw(6) << current.second.tax
        << " | " << std::setw(17) << current.second.workDaysCountDone
        << " | " << std::setw(13) << current.second.workDaysCount
        << " | " << std::setw(8) << current.second.received
        << " | " << std::setw(8) << current.second.withheld
        << " | " << std::setw(6) << current.second.gender
        << " |" << std::endl;
  };

  std::for_each(dataTable.begin(), dataTable.end(), serializeRowOperation);
  outputFileStream.close();

  std::cout << "Serialize complete\n";
}
void DBPerson::AppendDataToFile(const std::vector<Person> &listOfPersons) {
  std::ofstream appendFileStream(filename, std::ios::binary | std::ios::app);
  for (const auto &currentPerson: listOfPersons) {
    appendFileStream.write(reinterpret_cast<const char *>(&currentPerson), sizeof(Person));
  }

  std::cout << "Append complete\n";
  appendFileStream.close();
}
void DBPerson::RemoveDataFromFile(int id) {
  LoadData();
  if (id < 0 || id >= dataTable.size()) {
    std::cout << "Incorrect index\n";
  } else {
    dataTable.erase(id);
    size_t lastindex = filename.find_last_of('.');
    std::string bufferFilename;
    if (lastindex != std::string::npos) {
      auto rawname = filename.substr(0, lastindex);
      auto extenction = filename.substr(lastindex);
      bufferFilename = filename + "_removed" + extenction;
    } else {
      bufferFilename = filename + "_removed";
    }

    std::ofstream afterRemoveStream(bufferFilename, std::ios::binary | std::ios::out);
    for (const auto &elem: dataTable) {
      afterRemoveStream.write(reinterpret_cast<const char *>(&elem.second), sizeof(Person));
    }

    afterRemoveStream.close();
    std::cout << "Remove completed\n";
  }
}
void DBPerson::ReplaceDataInFile(int id, const Person &newPerson) {
  LoadData();
  if (id < 0 || id >= dataTable.size()) {
    std::cout << "Incorrect index\n";
  } else {
    size_t lastindex = filename.find_last_of('.');
    std::string bufferFilename;
    if (lastindex != std::string::npos) {
      auto rawname = filename.substr(0, lastindex);
      auto extenction = filename.substr(lastindex);
      bufferFilename = filename + "_replaced" + extenction;
    } else {
      bufferFilename = filename + "_replaced";
    }
    std::ofstream afterReplaceStream(bufferFilename, std::ios::binary);
    dataTable[id] = newPerson;
    for (const auto &row: dataTable)
      afterReplaceStream.write(reinterpret_cast<const char *>(&row.second), sizeof(Person));
    afterReplaceStream.close();

    std::cout << "Replacement completed\n";
  }
}


